/*
 * ====================================================================
 *
 * MIT License
 * 
 * Copyright (c) [2022] 
 * [Jan Grossniklaus, Jan Kunert, Sascha Hildebrands, Kevin Hinrichs]
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, 
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * urnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall
 * be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * ====================================================================
 */

/*
 + --------------------------------------------------------------------
 + 
 + Created    :  18/04/2022
 + Owner      :  Kevin Hinrichs
 +
 + Description:  Musik spectrum analyzer make cool effects
 +               on a LED stripe Matrix to music.
 +
 + --------------------------------------------------------------------
 */

#define EXTERN

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <arm_math.h>

#include "project.h" // global header file

#include "tm4c123gxl_init.h"
#include "ws2812b.h"

#include "inc/hw_memmap.h"

#include "driverlib/gpio.h"

//*********************************************************************
//
// Info
// Version: RC 2.01
// SRAM:
//  2.048 Bytes SRAM --> system stack
//  1.024 Bytes SRAM --> uDMA control table
// 12.481 Bytes SRAM --> led driver
// -----------------
// 15.523 Bytes SRAM
//
//*********************************************************************

//*********************************************************************
// 
// Hardware SETUP
//
// LED DATA SIGNAL 1: PB6
// LED DATA SIGNAL 2: PB4
// LED DATA SIGNAL 3: PB0
// LED DATA SIGNAL 4: PB2
//
// Need to be bridged: PB7 <---> PA7
// Need to be bridged: PB5 <---> PC7
// Need to be bridged: PB1 <---> PD3
// Need to be bridged: PB3 <---> PF2
//
//*********************************************************************

//*********************************************************************
//
// MAIN
//
//*********************************************************************
void main()
{
    //*****************************************************************
    //
    // Locale Variable
    //
    //*****************************************************************
    static streamDataDataTyp streamData; // 12.481 Bytes SRAM
    
    //*****************************************************************
    //
    // Hardware Main Setup
    //
    //*****************************************************************
    init_hardware();
    
    //*****************************************************************
    //
    // Debug Mode
    //
    //*****************************************************************
    #ifndef DEBUG_OFF
    init_GPIO_DEBUG();
    
    // reset all debug pins
    DEBUG_PA2 = 0;
    DEBUG_PA3 = 0;
    DEBUG_PA4 = 0;
    DEBUG_PA5 = 0;
    DEBUG_PA6 = 0;

    DEBUG_PC4 = 0;
    DEBUG_PC5 = 0;
    DEBUG_PC6 = 0;

    DEBUG_PE0 = 0;
    DEBUG_PE1 = 0;
    DEBUG_PE2 = 0;
    DEBUG_PE3 = 0;
    DEBUG_PE4 = 0;
    DEBUG_PE5 = 0;

    DEBUG_PF0 = 0;
    DEBUG_PF1 = 0;
    DEBUG_PF3 = 0;
    DEBUG_PF4 = 0;
    
    //
    // ***LA2016 - LED-Signal***
    //
    // DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    // DEBUG_PA5 = GPIO_PIN_5; // CH03 Pri
    
    // DEBUG_PA6 = GPIO_PIN_6; // CH04 Alt
    // DEBUG_PC4 = GPIO_PIN_4; // CH05 Color
    // DEBUG_PC5 = GPIO_PIN_5; // CH06 DMAP
    // DEBUG_PC6 = GPIO_PIN_6; // CH07 DMAA
    
    // PB6 -> CH08 // LED1
    // PB4 -> CH09 // LED2
    // PB0 -> CH10 // LED3
    // PB2 -> CH11 // LED4
    
    // PB7 -> CH12 // DMA1
    // PB5 -> CH13 // DMA2
    // PB1 -> CH14 // DMA3
    // PB3 -> CH15 // DMA4
    
    
    //
    // ***LA5016 - DSP***
    //
    // DEBUG_PE0 = GPIO_PIN_0;  // CH02 Pri
    // DEBUG_PE1 = GPIO_PIN_1;  // CH03 Alt
    // DEBUG_PE2 = GPIO_PIN_2;  // CH04 Mic
    // DEBUG_PE3 = GPIO_PIN_3;  // CH05 DMA P
    
    // DEBUG_PE4 = GPIO_PIN_4;  // CH06 DMA A
    // DEBUG_PE5 = GPIO_PIN_5;  // CH07 AGC
    // DEBUG_PF0 = GPIO_PIN_0;  // CH08 FFT
    // DEBUG_PF1 = GPIO_PIN_1;  // CH09 FBand
    
    // DEBUG_PF3 = GPIO_PIN_3;  // CH10
    // DEBUG_PF4 = GPIO_PIN_4;  // CH11
    // CH12
    // CH13
    // CH14
    // CH15
    
    //
    // ***LA2016 / LA5016***
    //
    // DEBUG_PA2 = GPIO_PIN_2; // CH00 Start LED
    // DEBUG_PA3 = GPIO_PIN_3; // CH01 Start Rec
    #endif
    
    streamData.sendLedData = 1;
    
    // switch all leds with full white
    for (int i = 0; i < 464; i++)
    {
        streamData.pixelData[i][0] = 255;
        streamData.pixelData[i][1] = 255;
        streamData.pixelData[i][2] = 255;
    }
    
    while (1)
    {
        
        if (streamData.sendLedData)
        {
            if (sendLED(&streamData))
            {
                streamData.sendLedData--;
            }
        }
    }
}